@@ -237,7 +237,7 @@ append_actions_menu (GtkWidget *menu_bar)
 	}
 
 	if (panel_is_program_in_path ("gnome-panel-screenshot")) {
-		item = pixmap_menu_item_new (_("Take a Screen Shot..."), "gnome-mdi.png", FALSE);
+		item = pixmap_menu_item_new (_("Take a Screen Shot..."), "gnome-screenshot.png", FALSE);
 		gtk_tooltips_set_tip (panel_tooltips, item,
 			      	      _("Take a screen shot of your desktop"),
 			              NULL);
@@ -323,7 +323,7 @@ programs_menu_to_display (GtkWidget *menu)
 			gtk_widget_destroy (GTK_MENU_SHELL (menu)->children->data);
 
 		create_root_menu (menu, TRUE, FOOBAR_MENU_FLAGS,
-				  FALSE, FALSE /* run_item */);
+				  FALSE, FALSE /* extra_items */);
 	}
 }
 
@@ -388,7 +388,7 @@ add_window (WnckWindow *window, FoobarWidget *foo)
 	GtkWidget *image = NULL;
 	GdkPixbuf *pb;
 	const char *name;
-	WnckScreen *screen = wnck_screen_get (0 /* FIXME screen number */);
+	WnckScreen *screen = wnck_screen_get_default ();
 	WnckWorkspace *wspace = wnck_screen_get_active_workspace (screen);
 
 	g_assert (foo->windows != NULL);
@@ -468,7 +468,7 @@ create_task_menu (GtkWidget *w, gpointer data)
 	FoobarWidget *foo = FOOBAR_WIDGET (data);
 	GList *list;
 	GtkWidget *separator;
-	WnckScreen *screen = wnck_screen_get (0 /* FIXME screen number */);
+	WnckScreen *screen = wnck_screen_get_default ();
 	GList *windows = wnck_screen_get_windows (screen);
 
 	/* g_message ("creating..."); */
@@ -513,46 +513,45 @@ destroy_task_menu (GtkWidget *w, gpointer data)
 }
 
 static void
-set_das_pixmap (FoobarWidget *foo, WnckWindow *window)
+set_das_pixmap (FoobarWidget *foo,
+		WnckWindow   *window)
 {
-	GdkPixbuf *pb;
-	if ( ! GTK_WIDGET_REALIZED (foo))
-		return;
+	GdkPixbuf *pixbuf = NULL;
 
-	if (foo->task_image != NULL)
-		gtk_widget_destroy (GTK_WIDGET (foo->task_image));
-	foo->task_image = NULL;
+	if (!GTK_WIDGET_REALIZED (foo))
+		return;
 
 	foo->icon_window = window;
 
-	pb = NULL;
-	if (window != NULL)
-		pb = wnck_window_get_mini_icon (window);
-	if (pb == NULL)
-		pb = get_default_image ();
+	if (window)
+		pixbuf = wnck_window_get_mini_icon (window);
 
-	if (pb != NULL) {
+	if (!pixbuf)
+		pixbuf = get_default_image ();
+
+	if (pixbuf) {
 		double pix_x, pix_y;
-		pix_x = gdk_pixbuf_get_width (pb);
-		pix_y = gdk_pixbuf_get_height (pb);
-		if (pix_x > ICON_SIZE || pix_y > ICON_SIZE) {
-			double greatest;
+
+		pix_x = gdk_pixbuf_get_width (pixbuf);
+		pix_y = gdk_pixbuf_get_height (pixbuf);
+
+		if (pix_x <= ICON_SIZE && pix_y <= ICON_SIZE) 
+			gtk_image_set_from_pixbuf (
+				GTK_IMAGE (foo->task_image), pixbuf);
+		else {
 			GdkPixbuf *scaled;
+			double     greatest;
 
 			greatest = pix_x > pix_y ? pix_x : pix_y;
-			scaled = gdk_pixbuf_scale_simple (pb,
-							  (ICON_SIZE / greatest) * pix_x,
-							  (ICON_SIZE / greatest) * pix_y,
-							  GDK_INTERP_BILINEAR);
-			foo->task_image = gtk_image_new_from_pixbuf (scaled);
-			g_object_unref (G_OBJECT (scaled));
-		} else {
-			foo->task_image = gtk_image_new_from_pixbuf (pb);
+			scaled = gdk_pixbuf_scale_simple (
+					pixbuf,
+					(ICON_SIZE / greatest) * pix_x,
+					(ICON_SIZE / greatest) * pix_y,
+					GDK_INTERP_BILINEAR);
+			gtk_image_set_from_pixbuf (
+				GTK_IMAGE (foo->task_image), scaled);
+			g_object_unref (scaled);
 		}
-		gtk_widget_show (foo->task_image);
-
-		gtk_container_add (GTK_CONTAINER (foo->task_bin),
-				   GTK_WIDGET (foo->task_image));
 	}
 }
 
@@ -567,6 +566,10 @@ append_task_menu (FoobarWidget *foo, GtkMenuShell *menu_bar)
 	gtk_widget_show (foo->task_bin);
 	gtk_container_add (GTK_CONTAINER (foo->task_item), foo->task_bin);
 
+	foo->task_image = gtk_image_new ();
+	gtk_widget_show (foo->task_image);
+	gtk_container_add (GTK_CONTAINER (foo->task_bin), foo->task_image);
+
 	gtk_menu_shell_append (menu_bar, foo->task_item);
 
 	panel_stretch_events_to_toplevel (
@@ -648,7 +651,7 @@ setup_task_menu (FoobarWidget *foo)
 
 	set_the_task_submenu (foo, foo->task_item);
 
-	screen = wnck_screen_get (0 /* FIXME screen number */);
+	screen = wnck_screen_get_default ();
 
 	/* setup the pixmap to the focused task */
 	windows = wnck_screen_get_windows (screen);
@@ -747,7 +750,7 @@ foobar_widget_instance_init (FoobarWidget *foo)
 					 TRUE /* force_image */);
 
 	menu = create_root_menu (NULL, TRUE, FOOBAR_MENU_FLAGS,
-				 FALSE, FALSE /* run_item */);
+				 FALSE, FALSE /* extra_items */);
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), menu);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu_bar), menuitem);
 	g_signal_connect (G_OBJECT (menu), "show",
